@@ -13,7 +13,7 @@
 namespace Composer\Test;
 
 use Composer\Console\Application;
-use Composer\TestCase;
+use Composer\Test\TestCase;
 use Symfony\Component\Console\Output\OutputInterface;
 
 class ApplicationTest extends TestCase
@@ -22,8 +22,8 @@ class ApplicationTest extends TestCase
     {
         $application = new Application;
 
-        $inputMock = $this->getMock('Symfony\Component\Console\Input\InputInterface');
-        $outputMock = $this->getMock('Symfony\Component\Console\Output\OutputInterface');
+        $inputMock = $this->getMockBuilder('Symfony\Component\Console\Input\InputInterface')->getMock();
+        $outputMock = $this->getMockBuilder('Symfony\Component\Console\Output\OutputInterface')->getMock();
 
         $index = 0;
         $inputMock->expects($this->at($index++))
@@ -31,6 +31,11 @@ class ApplicationTest extends TestCase
             ->with($this->equalTo('--no-plugins'))
             ->will($this->returnValue(true));
 
+        $inputMock->expects($this->at($index++))
+            ->method('hasParameterOption')
+            ->with($this->equalTo('--no-cache'))
+            ->will($this->returnValue(false));
+
         $inputMock->expects($this->at($index++))
             ->method('getParameterOption')
             ->with($this->equalTo(array('--working-dir', '-d')))
@@ -75,8 +80,8 @@ class ApplicationTest extends TestCase
 
         $application->add(new \Composer\Command\SelfUpdateCommand);
 
-        $inputMock = $this->getMock('Symfony\Component\Console\Input\InputInterface');
-        $outputMock = $this->getMock('Symfony\Component\Console\Output\OutputInterface');
+        $inputMock = $this->getMockBuilder('Symfony\Component\Console\Input\InputInterface')->getMock();
+        $outputMock = $this->getMockBuilder('Symfony\Component\Console\Output\OutputInterface')->getMock();
 
         $index = 0;
         $inputMock->expects($this->at($index++))
@@ -84,6 +89,11 @@ class ApplicationTest extends TestCase
             ->with($this->equalTo('--no-plugins'))
             ->will($this->returnValue(true));
 
+        $inputMock->expects($this->at($index++))
+            ->method('hasParameterOption')
+            ->with($this->equalTo('--no-cache'))
+            ->will($this->returnValue(false));
+
         $inputMock->expects($this->at($index++))
             ->method('getParameterOption')
             ->with($this->equalTo(array('--working-dir', '-d')))
