@@ -16,8 +16,9 @@ use Composer\Package\Version\VersionSelector;
 use Composer\Package\Package;
 use Composer\Package\Link;
 use Composer\Semver\VersionParser;
+use PHPUnit\Framework\TestCase;
 
-class VersionSelectorTest extends \PHPUnit_Framework_TestCase
+class VersionSelectorTest extends TestCase
 {
     // A) multiple versions, get the latest one
     // B) targetPackageVersion will pass to pool
@@ -197,7 +198,7 @@ class VersionSelectorTest extends \PHPUnit_Framework_TestCase
         $versionSelector = new VersionSelector($pool);
         $versionParser = new VersionParser();
 
-        $package = $this->getMock('\Composer\Package\PackageInterface');
+        $package = $this->getMockBuilder('\Composer\Package\PackageInterface')->getMock();
         $package
             ->expects($this->any())
             ->method('getPrettyVersion')
@@ -274,6 +275,6 @@ class VersionSelectorTest extends \PHPUnit_Framework_TestCase
 
     private function createMockPool()
     {
-        return $this->getMock('Composer\DependencyResolver\Pool', array(), array(), '', true);
+        return $this->getMockBuilder('Composer\DependencyResolver\Pool')->getMock();
     }
 }
