@@ -13,8 +13,9 @@
 namespace Composer\Test\Util;
 
 use Composer\Util\StreamContextFactory;
+use PHPUnit\Framework\TestCase;
 
-class StreamContextFactoryTest extends \PHPUnit_Framework_TestCase
+class StreamContextFactoryTest extends TestCase
 {
     protected function setUp()
     {
@@ -22,6 +23,7 @@ class StreamContextFactoryTest extends \PHPUnit_Framework_TestCase
         unset($_SERVER['http_proxy']);
         unset($_SERVER['HTTPS_PROXY']);
         unset($_SERVER['https_proxy']);
+        unset($_SERVER['NO_PROXY']);
         unset($_SERVER['no_proxy']);
     }
 
@@ -31,6 +33,7 @@ class StreamContextFactoryTest extends \PHPUnit_Framework_TestCase
         unset($_SERVER['http_proxy']);
         unset($_SERVER['HTTPS_PROXY']);
         unset($_SERVER['https_proxy']);
+        unset($_SERVER['NO_PROXY']);
         unset($_SERVER['no_proxy']);
     }
 
